"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var cp = require("child_process");
var path = require("path");
var logger_1 = require("../logger");
var pact_environment_1 = require("../pact-environment");
var arguments_1 = require("./arguments");
var _ = require('underscore');
var Spawn = (function () {
    function Spawn() {
    }
    Object.defineProperty(Spawn.prototype, "cwd", {
        get: function () {
            return path.resolve(__dirname, '..');
        },
        enumerable: true,
        configurable: true
    });
    Spawn.prototype.spawnBinary = function (command, args, argMapping) {
        if (args === void 0) { args = {}; }
        if (argMapping === void 0) { argMapping = {}; }
        var envVars = JSON.parse(JSON.stringify(process.env));
        delete envVars['RUBYGEMS_GEMDEPS'];
        var file;
        var opts = {
            cwd: pact_environment_1.default.cwd,
            detached: !pact_environment_1.default.isWindows(),
            env: envVars,
        };
        var cmd = [command]
            .concat(arguments_1.default.toArgumentsArray(args, argMapping))
            .join(' ');
        var spawnArgs;
        if (pact_environment_1.default.isWindows()) {
            file = 'cmd.exe';
            spawnArgs = ['/s', '/c', cmd];
            opts.windowsVerbatimArguments = true;
        }
        else {
            cmd = "./" + cmd;
            file = '/bin/sh';
            spawnArgs = ['-c', cmd];
        }
        logger_1.default.debug("Starting pact binary with '" + _.flatten([
            file,
            spawnArgs,
            JSON.stringify(opts),
        ]) + "'");
        var instance = cp.spawn(file, spawnArgs, opts);
        instance.stdout.setEncoding('utf8');
        instance.stderr.setEncoding('utf8');
        instance.stdout.on('data', logger_1.default.debug.bind(logger_1.default));
        instance.stderr.on('data', logger_1.default.debug.bind(logger_1.default));
        instance.on('error', logger_1.default.error.bind(logger_1.default));
        instance.once('close', function (code) {
            if (code !== 0) {
                logger_1.default.warn("Pact exited with code " + code + ".");
            }
        });
        logger_1.default.info("Created '" + cmd + "' process with PID: " + instance.pid);
        return instance;
    };
    Spawn.prototype.killBinary = function (binary) {
        if (binary) {
            var pid = binary.pid;
            logger_1.default.info("Removing Pact with PID: " + pid);
            binary.removeAllListeners();
            try {
                pact_environment_1.default.isWindows()
                    ? cp.execSync("taskkill /f /t /pid " + pid)
                    : process.kill(-pid, 'SIGINT');
            }
            catch (e) {
                return false;
            }
        }
        return true;
    };
    return Spawn;
}());
exports.Spawn = Spawn;
exports.default = new Spawn();
//# sourceMappingURL=spawn.js.map